@@ -15,7 +15,7 @@ has missing_after => 86400;
 has remove_after  => 864000;
 has tasks         => sub { {} };
 
-our $VERSION = '1.09';
+our $VERSION = '1.10';
 
 sub add_task { ($_[0]->tasks->{$_[1]} = $_[2]) and return $_[0] }
 
@@ -48,8 +48,8 @@ sub new {
 
 sub perform_jobs {
   my $self   = shift;
-  my $worker = $self->worker->register;
-  while (my $job = $worker->dequeue(0)) { $job->perform }
+  my $worker = $self->worker;
+  while (my $job = $worker->register->dequeue(0)) { $job->perform }
   $worker->unregister;
 }
 
@@ -84,7 +84,7 @@ Minion - Job queue
   use Minion;
 
   # Connect to backend
-  my $minion = Minion->new(File => '/Users/sri/minion.db');
+  my $minion = Minion->new(File => '/home/sri/minion.db');
   my $minion = Minion->new(Pg   => 'postgresql://postgres@/test');
 
   # Add tasks
@@ -103,9 +103,9 @@ Minion - Job queue
   $minion->perform_jobs;
 
   # Build more sophisticated workers
-  my $worker = $minion->repair->worker->register;
+  my $worker = $minion->repair->worker;
   while (int rand 2) {
-    if (my $job = $worker->dequeue(5)) { $job->perform }
+    if (my $job = $worker->register->dequeue(5)) { $job->perform }
   }
   $worker->unregister;
 
@@ -274,7 +274,7 @@ return C<undef> if job does not exist.
 
 =head2 new
 
-  my $minion = Minion->new(File => '/Users/sri/minion.db');
+  my $minion = Minion->new(File => '/home/sri/minion.db');
 
 Construct a new L<Minion> object.
 
@@ -314,8 +314,6 @@ This is the class hierarchy of the L<Minion> distribution.
 
 =over 2
 
-=item * L<Mojolicious::Plugin::Minion>
-
 =item * L<Minion>
 
 =item * L<Minion::Backend>
@@ -338,6 +336,8 @@ This is the class hierarchy of the L<Minion> distribution.
 
 =item * L<Minion::Worker>
 
+=item * L<Mojolicious::Plugin::Minion>
+
 =back
 
 =head1 AUTHOR
