@@ -29,12 +29,10 @@ sub enqueue {
 
   my $db = $self->pg->db;
   return $db->query(
-    "insert into minion_jobs
-       (args, created, delayed, priority, retries, state, task)
-     values
-       (?, now(), (now() + (interval '1 second' * ?)), ?, ?, ?, ?)
+    "insert into minion_jobs (args, delayed, priority, task)
+     values (?, (now() + (interval '1 second' * ?)), ?, ?)
      returning id", {json => $args}, $options->{delay} // 0,
-    $options->{priority} // 0, 0, 'inactive', $task
+    $options->{priority} // 0, $task
   )->hash->{id};
 }
 
@@ -56,8 +54,7 @@ sub list_jobs {
   my ($self, $offset, $limit, $options) = @_;
 
   return $self->pg->db->query(
-    'select id
-     from minion_jobs
+    'select id from minion_jobs
      where (state = $1 or $1 is null) and (task = $2 or $2 is null)
      order by id desc
      limit $3
@@ -68,10 +65,9 @@ sub list_jobs {
 sub list_workers {
   my ($self, $offset, $limit) = @_;
 
-  return $self->pg->db->query(
-    'select id from minion_workers order by id desc limit ? offset ?',
-    $limit, $offset)->arrays->map(sub { $self->worker_info($_->[0]) })
-    ->to_array;
+  my $sql = 'select id from minion_workers order by id desc limit ? offset ?';
+  return $self->pg->db->query($sql, $limit, $offset)
+    ->arrays->map(sub { $self->worker_info($_->[0]) })->to_array;
 }
 
 sub new {
@@ -83,11 +79,14 @@ sub new {
 }
 
 sub register_worker {
-  shift->pg->db->query(
-    'insert into minion_workers (host, pid, started)
-     values (?, ?, now())
-     returning id', hostname, $$
-  )->hash->{id};
+  my ($self, $id) = @_;
+
+  my $sql
+    = 'update minion_workers set notified = now() where id = ? returning 1';
+  return $id if $id && $self->pg->db->query($sql, $id)->rows;
+
+  $sql = 'insert into minion_workers (host, pid) values (?, ?) returning id';
+  return $self->pg->db->query($sql, hostname, $$)->hash->{id};
 }
 
 sub remove_job {
@@ -101,29 +100,28 @@ sub remove_job {
 sub repair {
   my $self = shift;
 
-  # Check workers on this host (all should be owned by the same user)
-  my $db   = $self->pg->db;
-  my $host = hostname;
-  my $workers
-    = $db->query('select id, pid from minion_workers where host = ?', $host)
-    ->hashes;
-  my @dead = map { $_->{id} } grep { !kill 0, $_->{pid} } $workers->each;
-  $db->query("delete from minion_workers where id = any (?)", \@dead) if @dead;
+  # Check worker registry
+  my $db     = $self->pg->db;
+  my $minion = $self->minion;
+  $db->query(
+    "delete from minion_workers
+     where notified < now() - interval '1 second' * ?", $minion->missing_after
+  );
 
   # Abandoned jobs
   $db->query(
     "update minion_jobs as j
-     set result = to_json('Worker not found by ' || ?), state = 'failed'
+     set finished = now(), result = to_json('Worker went away'::text),
+       state = 'failed'
      where state = 'active'
-       and not exists(select 1 from minion_workers where id = j.worker)",
-    "$host:$$"
+       and not exists(select 1 from minion_workers where id = j.worker)"
   );
 
   # Old jobs
   $db->query(
     "delete from minion_jobs
      where state = 'finished' and finished < now() - interval '1 second' * ?",
-    $self->minion->remove_after
+    $minion->remove_after
   );
 }
 
@@ -137,8 +135,7 @@ sub retry_job {
   !!shift->pg->db->query(
     "update minion_jobs
      set finished = null, result = null, retried = now(),
-       retries = retries + 1, started = null, state = 'inactive',
-       worker = null
+       retries = retries + 1, started = null, state = 'inactive', worker = null
      where id = ? and state in ('failed', 'finished')
      returning 1", shift
   )->rows;
@@ -147,16 +144,14 @@ sub retry_job {
 sub stats {
   my $self = shift;
 
-  my $db     = $self->pg->db;
-  my $all    = $db->query('select count(*) from minion_workers')->array->[0];
-  my $active = $db->query(
-    "select count(distinct worker)
-     from minion_jobs
-     where state = 'active'"
-  )->array->[0];
+  my $db  = $self->pg->db;
+  my $all = $db->query('select count(*) from minion_workers')->array->[0];
+  my $sql
+    = "select count(distinct worker) from minion_jobs where state = 'active'";
+  my $active = $db->query($sql)->array->[0];
 
-  my $states
-    = $db->query('select state, count(state) from minion_jobs group by 1')
+  $sql = 'select state, count(state) from minion_jobs group by 1';
+  my $states = $db->query($sql)
     ->arrays->reduce(sub { $a->{$b->[0]} = $b->[1]; $a }, {});
 
   return {
@@ -175,11 +170,12 @@ sub unregister_worker {
 
 sub worker_info {
   shift->pg->db->query(
-    'select id, array(
-       select id from minion_jobs where worker = minion_workers.id
+    "select id, extract(epoch from notified) as notified, array(
+       select id from minion_jobs
+       where state = 'active' and worker = minion_workers.id
      ) as jobs, host, pid, extract(epoch from started) as started
      from minion_workers
-     where id = ?', shift
+     where id = ?", shift
   )->hash;
 }
 
@@ -189,16 +185,14 @@ sub _try {
   return $self->pg->db->query(
     "update minion_jobs
      set started = now(), state = 'active', worker = ?
-     from (
-       select id
-       from minion_jobs
+     where id = (
+       select id from minion_jobs
        where state = 'inactive' and delayed < now() and task = any (?)
        order by priority desc, created
        limit 1
        for update
-     ) job
-     where minion_jobs.id = job.id
-     returning minion_jobs.id, args, task", $id, [keys %{$self->minion->tasks}]
+     )
+     returning id, args, task", $id, [keys %{$self->minion->tasks}]
   )->expand->hash;
 }
 
@@ -231,8 +225,8 @@ Minion::Backend::Pg - PostgreSQL backend
 
 L<Minion::Backend::Pg> is a backend for L<Minion> based on L<Mojo::Pg>. All
 necessary tables will be created automatically with a set of migrations named
-C<minion>. Note that this backend uses many bleeding edge features, so only
-the latest, stable version of PostgreSQL is fully supported.
+C<minion>. Note that this backend uses many bleeding edge features, so only the
+latest, stable version of PostgreSQL is fully supported.
 
 =head1 ATTRIBUTES
 
@@ -346,8 +340,9 @@ Construct a new L<Minion::Backend::Pg> object.
 =head2 register_worker
 
   my $worker_id = $backend->register_worker;
+  my $worker_id = $backend->register_worker($worker_id);
 
-Register worker.
+Register worker or send heartbeat to show that this worker is still alive.
 
 =head2 remove_job
 
@@ -439,3 +434,11 @@ create trigger minion_jobs_insert_trigger after insert on minion_jobs
 drop table if exists minion_jobs;
 drop function if exists minion_jobs_insert_notify();
 drop table if exists minion_workers;
+
+-- 2 up
+alter table minion_jobs alter column created set default now();
+alter table minion_jobs alter column state set default 'inactive';
+alter table minion_jobs alter column retries set default 0;
+alter table minion_workers add column
+  notified timestamp with time zone not null default now();
+alter table minion_workers alter column started set default now();
